"""
///
/// @file HabsoluteState.py
///
/// @brief AUTOGENERATED FILE! ONLY EDIT IF YOU ARE A MACHINE!
///
/// @core c4674f111fc72bad86b69ef5e7f862f75f59d86f
///
/// @build 6a54779ef7f5d112e58f4877e4667f33623592a3
///
/// @date 2023-02-23 14:31:03
///
/// @author Dephy, Inc.
"""
# pylint: disable=duplicate-code

from ctypes import Structure, c_int


class HabsoluteState(Structure):
    _pack_ = 1
    _fields_ = [
        ("device_family", c_int),
        ("variable_label", c_int),
        ("include files:", c_int),
        ("state_time", c_int),
        ("ank_ang", c_int),
        ("ank_vel", c_int),
        ("adc_0", c_int),
        ("adc_1", c_int),
        ("adc_2", c_int),
        ("adc_3", c_int),
        ("adc_4", c_int),
        ("adc_5", c_int),
        ("adc_6", c_int),
        ("adc_7", c_int),
        ("genvar_0", c_int),
        ("genvar_1", c_int),
        ("genvar_2", c_int),
        ("genvar_3", c_int),
        ("status", c_int),
        ("SystemTime", c_int),
    ]



"""
///
/// @file BMSState.py
///
/// @brief AUTOGENERATED FILE! ONLY EDIT IF YOU ARE A MACHINE!
///
/// @core c4674f111fc72bad86b69ef5e7f862f75f59d86f
///
/// @build 6a54779ef7f5d112e58f4877e4667f33623592a3
///
/// @date 2023-02-23 14:31:03
///
/// @author Dephy, Inc.
"""
# pylint: disable=duplicate-code

from ctypes import Structure, c_int


class BMSState(Structure):
    _pack_ = 1
    _fields_ = [
        ("device_family", c_int),
        ("variable_label", c_int),
        ("include files:", c_int),
        ("state_time", c_int),
        ("cells_0_mv", c_int),
        ("cells_1_mv", c_int),
        ("cells_2_mv", c_int),
        ("cells_3_mv", c_int),
        ("cells_4_mv", c_int),
        ("cells_5_mv", c_int),
        ("cells_6_mv", c_int),
        ("cells_7_mv", c_int),
        ("cells_8_mv", c_int),
        ("bms_state", c_int),
        ("error", c_int),
        ("current", c_int),
        ("stack_voltage", c_int),
        ("temperature_0", c_int),
        ("temperature_1", c_int),
        ("temperature_2", c_int),
        ("pack_imbalance", c_int),
        ("balancing", c_int),
        ("timer", c_int),
        ("user_interface", c_int),
        ("bq_sys_stat", c_int),
        ("bq_sys_ctrl1", c_int),
        ("bq_sys_ctrl2", c_int),
        ("fw_version", c_int),
        ("genvar_0_", c_int),
        ("genvar_1_", c_int),
        ("SystemTime", c_int),
    ]



# pylint: disable=duplicate-code

import argparse
from time import sleep

import numpy as np
from utils import clear  # pylint: disable=import-error

from flexsea.device import Device


# ============================================
#                    main
# ============================================
def main(
    port: str,
    cLibVersion: str,
    libFile: str,
    minVoltage: int,
    maxVoltage: int,
    nVoltages: int,
    nCycles: int,
    freq: int,
) -> None:
    device = Device(port=port, cLibVersion=cLibVersion, libFile=libFile)
    device.open()
    device.start_streaming(freq)

    voltages = np.linspace(minVoltage, maxVoltage, nVoltages)

    for _ in range(nCycles):
        # Ramp up
        for voltage in voltages:
            device.command_motor_voltage(int(voltage))
            sleep(0.1)
            clear()
            device.print()

        # Ramp down
        for voltage in voltages[-1::-1]:
            device.command_motor_voltage(int(voltage))
            sleep(0.1)
            clear()
            device.print()

    device.close()


# ============================================
#                  Run Main
# ============================================
if __name__ == "__main__":
    parser = argparse.ArgumentParser()
    parser.add_argument(
        "-p",
        "--port",
        dest="port",
        type=str,
        default="",
        help="Serial port device is connected to.",
    )
    parser.add_argument(
        "-c",
        "--clibversion",
        dest="cLibVersion",
        type=str,
        default="",
        help="Semantic version string of the pre-compiled C library.",
    )
    parser.add_argument(
        "-l",
        "--libfile",
        dest="libFile",
        type=str,
        default="",
        help="Path to the pre-compiled C library to use.",
    )
    parser.add_argument(
        "--max-voltage",
        dest="maxVoltage",
        type=int,
        default=3000,
        help="Maximum voltage to ramp to, in millivolts.",
    )
    parser.add_argument(
        "--min-voltage",
        dest="minVoltage",
        type=int,
        default=0,
        help="Minium voltage to start at, in millivolts.",
    )
    parser.add_argument(
        "-n",
        "--n-voltages",
        dest="nVoltages",
        type=int,
        default=10,
        help="The number of voltages to ramp over.",
    )
    parser.add_argument(
        "-i",
        "--iterations",
        dest="nCycles",
        type=int,
        default=5,
        help="The number of times to loop over the voltage range.",
    )
    parser.add_argument(
        "-f",
        "--frequency",
        dest="freq",
        type=int,
        default=100,
        help="The frequency (Hz) at which to stream data from the device.",
    )

    args = parser.parse_args()

    main(
        args.port,
        args.cLibVersion,
        args.libFile,
        args.minVoltage,
        args.maxVoltage,
        args.nVoltages,
        args.nCycles,
        args.freq,
    )

    print("Done.")



# pylint: disable=duplicate-code

import argparse
from time import sleep

import numpy as np
from utils import plot  # pylint: disable=import-error

from flexsea.device import Device


# ============================================
#                    main
# ============================================
def main(  # pylint: disable=too-many-locals
    port: str,
    cLibVersion: str,
    libFile: str,
    freq: int,
    gains: dict,
    waveAmplitude: int,
    waveFrequency: int,
    commandFrequency: int,
    nLoops: int,
    cycleDelay: float,
):
    device = Device(port=port, cLibVersion=cLibVersion, libFile=libFile)
    device.open()

    device.start_streaming(freq)

    gains = {
        "kp": 40,
        "ki": 400,
        "kd": 0,
        "k": 0,
        "b": 0,
        "ff": 128,
    }
    device.set_gains(**gains)

    nSamples = int(commandFrequency / waveFrequency)
    x = np.linspace(-np.pi, np.pi, nSamples)
    currents = waveAmplitude * np.sin(x)

    commandDelay = 1 / commandFrequency
    cycleDelay = 0.1

    measuredCurrent = []
    desiredCurrent = []
    deviceTime = []

    for _ in range(nLoops):
        for current in currents:
            sleep(commandDelay)

            data = device.read()

            measuredCurrent.append(data["mot_cur"])
            desiredCurrent.append(current)
            deviceTime.append(data["state_time"])

            device.command_motor_current(int(current))

            # Delay between cycles
            for __ in range(int(cycleDelay / commandDelay)):
                sleep(commandDelay)

                data = device.read()

                measuredCurrent.append(data["mot_cur"])
                desiredCurrent.append(current)
                deviceTime.append(data["state_time"])

    device.close()
    print("Plotting...")
    plot(desiredCurrent, measuredCurrent, deviceTime, "Current (mA)", "high_speed.png")


# ============================================
#                  Run Main
# ============================================
if __name__ == "__main__":
    parser = argparse.ArgumentParser()

    parser.add_argument(
        "-p",
        "--port",
        dest="port",
        type=str,
        default="",
        help="Serial port device is connected to.",
    )
    parser.add_argument(
        "-c",
        "--clibversion",
        dest="cLibVersion",
        type=str,
        default="",
        help="Semantic version string of the pre-compiled C library.",
    )
    parser.add_argument(
        "-l",
        "--libfile",
        dest="libFile",
        type=str,
        default="",
        help="Path to the pre-compiled C library to use.",
    )
    parser.add_argument(
        "-f",
        "--frequency",
        dest="freq",
        type=int,
        default=100,
        help="Frequency (Hz) at which device will stream data.",
    )
    parser.add_argument(
        "--kp",
        dest="kp",
        type=int,
        default=40,
        help="Proportional gain.",
    )
    parser.add_argument(
        "--ki",
        dest="ki",
        type=int,
        default=400,
        help="Integral gain.",
    )
    parser.add_argument(
        "--kd",
        dest="kd",
        type=int,
        default=0,
        help="Differential gain.",
    )
    parser.add_argument(
        "--k",
        dest="k",
        type=int,
        default=0,
        help="Stiffness gain.",
    )
    parser.add_argument(
        "--b",
        dest="b",
        type=int,
        default=0,
        help="Damping gain.",
    )
    parser.add_argument(
        "--ff",
        dest="ff",
        type=int,
        default=128,
        help="Feed-forward gain.",
    )
    parser.add_argument(
        "--wave-frequency",
        dest="waveFrequency",
        type=int,
        default=5,
        help="Frequency of the sine wave for the motor currents.",
    )
    parser.add_argument(
        "--wave-amplitude",
        dest="waveAmplitude",
        type=int,
        default=500,
        help="Amplitude of the sine wave for the motor currents.",
    )
    parser.add_argument(
        "--command-frequency",
        dest="commandFrequency",
        type=int,
        default=100,
        help="Reciprocal of the time between motor commands.",
    )
    parser.add_argument(
        "--n-loops",
        dest="nLoops",
        type=int,
        default=1,
        help="Number of times to iterate.",
    )
    parser.add_argument(
        "--cycle-delay",
        dest="cycleDelay",
        type=float,
        default=0.1,
        help="Delay between loops.",
    )

    args = parser.parse_args()

    gains_dict = {
        "kp": args.kp,
        "ki": args.ki,
        "kd": args.kd,
        "k": args.k,
        "b": args.b,
        "ff": args.ff,
    }

    main(
        args.port,
        args.cLibVersion,
        args.libFile,
        args.freq,
        gains_dict,
        args.waveAmplitude,
        args.waveFrequency,
        args.commandFrequency,
        args.nLoops,
        args.cycleDelay,
    )

    print("Done.")



"""
///
/// @file ../../Actuator-Package/flexsea/specs/device_specs/all_devices.py
///
/// @brief AUTOGENERATED FILE! ONLY EDIT IF YOU ARE A MACHINE!
///
/// @core c4674f111fc72bad86b69ef5e7f862f75f59d86f
///
/// @build 6a54779ef7f5d112e58f4877e4667f33623592a3
///
/// @date 2023-03-06
///
/// @author Dephy, Inc.
"""
# pylint: disable=unused-import

from .ActPackState import ActPackState
from .BMSState import BMSState
from .EB5xState import EB5xState
from .EB60State import EB60State
from .EB603State import EB603State
from .EB604State import EB604State
from .HabsoluteState import HabsoluteState
from .MD10State import MD10State
from .MD11State import MD11State
from .MD12State import MD12State
from .MD20State import MD20State
from .MD30State import MD30State
from .MD31State import MD31State
from .NB20State import NB20State
from .NewBooState import NewBooState
from .XC00sState import XC00sState
from .XC00tState import XC00tState
from .XC01State import XC01State
from .XC02State import XC02State
from .XC03State import XC03State
